# encoding: utf-8
#
#= GraphstoresController
#
#Authors:: パーソナルメディア株式会社
#Copyright:: Copyrigyt (C) Personal Media Corp., 2013. All rights reserved.
#
require 'rdf/mongo'


#== /api/vi/rdf-graph-store/ に対応したコントローラ
class GraphstoresController < ApplicationController

  URIS = [
          ["fishery","http://fishery.suisancloud.net/ns/fishery#"],
          ["rdf","http://www.w3.org/1999/02/22-rdf-syntax-ns#"],
          ["rdfs","http://www.w3.org/2000/01/rdf-schema#"],
          ["fev","http://fishery.suisancloud.net/ns/fisheryevent#"],
          ["ev","http://uidcenter.org/vocab/ucr/event#"],
          ["dc","http://purl.org/dc/elements/1.1/"],
          ["uobj","http://uidcenter.org/vocab/ucr/uobj#"],
          ["foaf","http://xmlns.com/foaf/0.1/"],
          ["ug","http://uidcenter.org/vocab/ucr/ug#"],
          ["daml","http://www.daml.org/2001/03/daml+oil#"],
          ["spac","http://uidcenter.org/vocab/ucr/spac#"],
          ["vcard","http://www.w3.org/2001/vcard-rdf/3.0#"],
          ["dcterms","http://purl.org/dc/terms/"],
          ["eg","http://www.example.org/"],
          ["trans","http://uidcenter.org/vocab/ucr/transaction#"],
          ["dctype","http://purl.org/dc/dcmitype/"],
          ["ugsrv","http://uidcenter.org/vocab/ucr/ugsrv#"],
          ["xsd","http://www.w3.org/2001/XMLSchema#"],
          ["owl","http://www.w3.org/2002/07/owl#"],
          ["rss","http://purl.org/rss/1.0/"],
          ["uc","http://uidcenter.org/vocab/ucr/uc#"],
          ["ag","http://www.agri-info.jp/vocab/ag#"],
          ["ja","http://jena.hpl.hp.com/2005/11/Assembler#"],
#         ["ucode","urn:ucode:"],
          ["geof","http://www.opengis.net/def/function/geosparql/"],
          ["geo","http://www.opengis.net/ont/geosparql#"],
          ["ucr","http://www.jibaninfo.jp/ucr/ucr#"],
          ["w3cgeo","http://www.w3.org/2003/01/geo/wgs84_pos#"],
          ["sf","http://www.opengis.net/ont/sf#"],
          ["ogc","http://www.opengis.net/"],
          ["geor","http://www.opengis.net/def/rule/geosparql/"],
          ["gml","http://www.opengis.net/ont/gml#"],
          ["gs","http://www.jibaninfo.jp/vocab/ucr/gs#"],
          ["puti","http://opendata.ubin.jp/puti#"]
         ]
  #=== /api/vi/rdf-graph-store/ GET メソッドを処理する
  def show
    uri = URI.parse("http://sesame.opendata.ubin.jp:8080/openrdf-sesame/repositories/opendata/rdf-graphs/service")
    query = nil
    if params[:graph].present?
      query = { 'graph' => params[:graph]}
    elsif params.key?("default")
      uri = URI.parse("http://sesame.opendata.ubin.jp:8080/openrdf-sesame/repositories/opendata/rdf-graphs/service?default")
    else
      head 400
    end
    http_client = HTTPClient.new
    header = [[ 'Accept', request.headers['ACCEPT'] ]]
    res = http_client.get(uri,query,header)
    response.headers['Content-Type'] = res.contenttype
    render :text => res.content

  end

  #=== /api/vi/rdf-graph-store/ POST メソッドを処理する
  def create
    uri = URI.parse("http://sesame.opendata.ubin.jp:8080/openrdf-sesame/repositories/opendata/statements")
    if params[:graph].present?
      uri = URI.parse("http://sesame.opendata.ubin.jp:8080/openrdf-sesame/repositories/opendata/rdf-graphs/service?graph=" + params[:graph])
    elsif params.key?("default")
      uri = URI.parse("http://sesame.opendata.ubin.jp:8080/openrdf-sesame/repositories/opendata/statements")
    else
      head 400 and return
    end

    http_client = HTTPClient.new
    header = [ ['Content-type', request.headers['CONTENT_TYPE'] ] ]

    # まず Repository statements に投げる
    res = http_client.post(uri,request.body.string,header)
    response.headers['Content-Type'] = res.contenttype


    # RDF.rb では RDF/XML -> n-triples -> RDF/JSON と変換して遅い。
    # SPARQL サーバから結果を x-tutle で受け取り、
    # x-tutle -> RDF/JSON を自前で行う。このほうが早い。

    # 投げた結果を x-turtle で受け取る
    header = [ ['Accept',  'application/x-turtle' ]]
    res = http_client.get(uri,nil,header)

    # x-turtle -> JSON 変換
    writer = RDF::Writer.for(:json)
    data = RDF::Reader.for(:turtle).new(res.body)

    sio = StringIO.new('')
    writer.dump( data, sio ) 
    json_str = sio.string; # JSON 文字列にして調整する
    # 短縮名を生成する
    URIS.each do |i|
      json_str.gsub!(i[1],i[0]+'_')
    end
    # データを文字列のまま整える
    json_str.gsub!("urn:ucode:_","ucode_")
    # MongoDB の ucode フィールドには32桁のucodeのみ投入する
    json_str.gsub!(/"ucode_([0-9a-fA-F]{32})":{/, '{"ucode":"\1",' )
    json_str.sub!(/}$/,'')
    json_str.sub!(/^{/,'')
    json_str.gsub!("]},{","]}\n---\n{")

    m = Mongo::Connection.new(MONGO_SERVER,MONGO_PORT)
    db = m.db(MONGO_DB)

    # JSON に戻してバルクインサート
    stream = YAML.load_stream(json_str)
    stream.each do |doc|
      db[MONGO_DB].insert(doc)
    end

    head :ok
  end

  #=== /api/vi/rdf-graph-store/ PUT メソッドを処理する
  def update
    uri = URI.parse("http://sesame.opendata.ubin.jp:8080/openrdf-sesame/repositories/opendata/statements")
    if params[:graph].present?
      uri = URI.parse("http://sesame.opendata.ubin.jp:8080/openrdf-sesame/repositories/opendata/rdf-graphs/service?graph=" + params[:graph])
    elsif params.key?("default")
      uri = URI.parse("http://sesame.opendata.ubin.jp:8080/openrdf-sesame/repositories/opendata/statements")
    else
      head 400 and return
    end

    http_client = HTTPClient.new
    header = [ ['Content-type', request.headers['CONTENT_TYPE'] ] ]

    # まず Repository statements に投げる
    res = http_client.put(uri,request.body.string,header)
    response.headers['Content-Type'] = res.contenttype

    # RDF.rb では RDF/XML -> n-triples -> RDF/JSON と変換して遅い。
    # SPARQL サーバから結果を x-tutle で受け取り、
    # x-tutle -> RDF/JSON を自前で行う。このほうが早い。

    # 投げた結果を x-turtle で受け取る
    header = [ ['Accept',  'application/x-turtle' ]]
    res = http_client.get(uri,nil,header)

    # x-turtle -> JSON 変換
    writer = RDF::Writer.for(:json)
    data = RDF::Reader.for(:turtle).new(res.body)

    sio = StringIO.new('')
    writer.dump( data, sio ) 
    json_str = sio.string; # JSON 文字列にして調整する
    # 短縮名を生成する
    URIS.each do |i|
      json_str.gsub!(i[1],i[0]+'_')
    end
    # データを文字列のまま整える
    json_str.gsub!("urn:ucode:_","ucode_")
    # MongoDB の ucode フィールドには32桁のucodeのみ投入する
    json_str.gsub!(/"ucode_([0-9a-fA-F]{32})":{/, '{"ucode":"\1",' )
    json_str.sub!(/}$/,'')
    json_str.sub!(/^{/,'')
    json_str.gsub!("]},{","]}\n---\n{")

    m = Mongo::Connection.new(MONGO_SERVER,MONGO_PORT)
    db = m.db(MONGO_DB)
    db[MONGO_DB].remove

    # JSON に戻してインサート
    stream = YAML.load_stream(json_str)
    stream.each do |doc|
      db[MONGO_DB].insert(doc)
    end

    head :ok
  end


  #=== /api/vi/rdf-graph-store/ DELETE メソッドを処理する
  def destroy
    uri = URI.parse("http://sesame.opendata.ubin.jp:8080/openrdf-sesame/repositories/opendata/rdf-graphs/service?default")
    if params[:graph].present?
      uri = URI.parse ("http://sesame.opendata.ubin.jp:8080/openrdf-sesame/repositories/opendata/rdf-graphs/service%3fgraph%3d" + params[:graph])
    elsif params.key?("default")
      uri = URI.parse("http://sesame.opendata.ubin.jp:8080/openrdf-sesame/repositories/opendata/rdf-graphs/service?default")
    end
    header = {}
    http_client = HTTPClient.new
    res = http_client.delete(uri,header)
    response.headers['Content-Type'] = res.contenttype


    m = Mongo::Connection.new(MONGO_SERVER,MONGO_PORT)
    db = m.db(MONGO_DB)
    db[MONGO_DB].remove


    render :nothing => true, :status => res.status_code
  end

end
