# coding: utf-8
#
#= RawdataController
#
# Authors:: パーソナルメディア株式会社
# Copyright:: Copyrigyt (C) Personal Media Corp., 2013. All rights reserved.
#

require 'java'
require '/usr/java/packages/lib/ext/commons-codec-1.7.jar'
require '/usr/java/packages/lib/ext/commons-httpclient-3.1.jar'
require '/usr/java/packages/lib/ext/commons-logging-1.1.1.jar'
require '/usr/java/packages/lib/ext/log4j-1.2.14.jar'
require '/usr/java/packages/lib/ext/openrdf-sesame-2.6.10-onejar.jar'
require '/usr/java/packages/lib/ext/slf4j-api-1.7.2.jar'
require '/usr/java/packages/lib/ext/slf4j-nop-1.7.2.jar'
require '/usr/java/packages/lib/ext/sesame.jar'

#== /api/vi/rawdata に対応したコントローラ
class RawdataController < ApplicationController


  URIS = [
          ["fishery","http://fishery.suisancloud.net/ns/fishery#"],
          ["rdf","http://www.w3.org/1999/02/22-rdf-syntax-ns#"],
          ["rdfs","http://www.w3.org/2000/01/rdf-schema#"],
          ["fev","http://fishery.suisancloud.net/ns/fisheryevent#"],
          ["ev","http://uidcenter.org/vocab/ucr/event#"],
          ["dc","http://purl.org/dc/elements/1.1/"],
          ["uobj","http://uidcenter.org/vocab/ucr/uobj#"],
          ["foaf","http://xmlns.com/foaf/0.1/"],
          ["ug","http://uidcenter.org/vocab/ucr/ug#"],
          ["daml","http://www.daml.org/2001/03/daml+oil#"],
          ["spac","http://uidcenter.org/vocab/ucr/spac#"],
          ["vcard","http://www.w3.org/2001/vcard-rdf/3.0#"],
          ["dcterms","http://purl.org/dc/terms/"],
          ["eg","http://www.example.org/"],
          ["trans","http://uidcenter.org/vocab/ucr/transaction#"],
          ["dctype","http://purl.org/dc/dcmitype/"],
          ["ugsrv","http://uidcenter.org/vocab/ucr/ugsrv#"],
          ["xsd","http://www.w3.org/2001/XMLSchema#"],
          ["owl","http://www.w3.org/2002/07/owl#"],
          ["rss","http://purl.org/rss/1.0/"],
          ["uc","http://uidcenter.org/vocab/ucr/uc#"],
          ["ag","http://www.agri-info.jp/vocab/ag#"],
          ["ja","http://jena.hpl.hp.com/2005/11/Assembler#"],
#         ["ucode","urn:ucode:"],
          ["geof","http://www.opengis.net/def/function/geosparql/"],
          ["geo","http://www.opengis.net/ont/geosparql#"],
          ["ucr","http://www.jibaninfo.jp/ucr/ucr#"],
          ["w3cgeo","http://www.w3.org/2003/01/geo/wgs84_pos#"],
          ["sf","http://www.opengis.net/ont/sf#"],
          ["ogc","http://www.opengis.net/"],
          ["geor","http://www.opengis.net/def/rule/geosparql/"],
          ["gml","http://www.opengis.net/ont/gml#"],
          ["gs","http://www.jibaninfo.jp/vocab/ucr/gs#"],
          ["puti","http://opendata.ubin.jp/puti#"]
         ]

  #=== /api/vi/rawdata GET メソッドを処理する
  def show

    # Java 用に引数を構築する
    array = []
    if params[:subject].present?
      array.push("-s" + params[:subject])
    end
    if params[:predicate].present?
      array.push("-p" + params[:predicate])
    end
    if params[:object].present?
      if /^http:\/\// =~ params[:object] 
        array.push("-o" + params[:object])
      elsf /^<http:\/\// =~ params[:object] 
        len = params[:object].length
        array.push("-o" + params[:object].slice(1,len-2))
      else
        array.push("-O" + params[:object])
      end
    end
    if params[:graph].present?
      array.push("-g" + params[:predicate])
    end

    if request.headers['ACCEPT'] == "text/plain"
      array.push("-n")
      response.headers["Content-Type"] = "text/plain"
    elsif request.headers['ACCEPT'] == "text/rdf+n3"
      array.push("-3")
      response.headers["Content-Type"] = "text/rdf+n3"
    else request.headers['ACCEPT'] == "application/rdf+xml"
      array.push("-x")
      response.headers["Content-Type"] = "application/rdf+xml"
    end

    array.push("-ropendata")
    res =  Java::pmc::Sesameget.results(array.to_java(:string))
    render :text => res
  end

  #=== /api/vi/rawdata POST メソッドを処理する
  def create

    # Java 用に引数を組み立てる
    array = []

    if params[:graph].present?
      array.push("-g" + params[:graph])
    end

    if request.headers['CONTENT_TYPE'] == "text/plain"
      array.push("-n" + request.body.string)
    else request.headers['CONTENT_TYPE'] == "application/rdf+xml"
      array.push("-x" + request.body.string)
    end

    array.push("-ropendata")

    res = 0
    # java で sesame に書き込む
    res =  Java::pmc::Sesamepostraw.results(array.to_java(:string))
    
    # 重複していればエラー
    if res == 1
      head 409 and return
    end


    # Mongo に書き込むためにSVOを抽出、加工
    ss = ''
    pp = ''
    oo = ''
    RDF::Reader.for(:rdfxml).new(request.body.string) do |reader|
      reader.each_statement do |statement|
        statement.subject.to_s =~ /([0-9a-fA-F]{32})/
        ss = $1 # MongoDB の ucode フィールドは32桁のucodeのみ
        pp = statement.predicate.to_s
        oo = statement.object.to_s
      end
    end

    URIS.each do |i|
      pp.sub!(i[1],i[0]+'_')
    end

    # 書き込み用ハッシュ作成
    
    data =
      { "type"  =>  "literal",
      "value" =>  oo 
    }  

    m = Mongo::Connection.new(MONGO_SERVER,MONGO_PORT)
    db = m.db(MONGO_COLLECTION)
#    db[MONGO_COLLECTION].update( {'ucode' => ss },{ '$set' => { pp => data } } )
    db[MONGO_COLLECTION].update( {'ucode' => ss },{ '$push' => { pp => data } } )
#    db[MONGO_COLLECTION].update( {'ucode' => ss },{ '$pop'  => { pp  => -1 } } )
    head 204

  end

  #=== /api/vi/rawdata PUT メソッドを処理する
  def update
    # RDF/XML ONLY
    idx = request.body.string.rindex("<?")
    len = request.body.string.length
    arg1 = request.body.string.slice(0..(idx-1))
    arg2 = request.body.string.slice(idx..len-1)

    array1 = []
    array2 = []

    if params[:graph].present?
      array1.push("-g" + params[:graph])
      array2.push("-g" + params[:graph])
    end

    if request.headers['CONTENT_TYPE'] == "text/plain"
      array1.push("-n" + arg1)
      array2.push("-n" + arg2)
    else request.headers['CONTENT_TYPE'] == "application/rdf+xml"
      array1.push("-x" + arg1)
      array2.push("-x" + arg2)
    end

    array1.push("-ropendata")
    array2.push("-ropendata")

    res = 0
    res =  Java::pmc::Sesamedelete.results(array1.to_java(:string))

    if res == 1
      head 404 and return
    end

    res = 0
    res =  Java::pmc::Sesamepost.results(array2.to_java(:string))
      
    if res == 1
      head 409 and return
    end


    # Mongo に書き込むためにSVOを抽出、加工
    ss = ''
    pp = ''
    oo = ''
    RDF::Reader.for(:rdfxml).new(arg1) do |reader|
      reader.each_statement do |statement|
        statement.subject.to_s =~ /([0-9a-fA-F]{32})/
        ss =  $1 # MongoDB の ucode フィールドは32桁のucodeのみ
        pp = statement.predicate.to_s
        oo = statement.object.to_s
      end
    end

    URIS.each do |i|
      pp.sub!(i[1],i[0]+'_')
    end

    # 削除用ハッシュ作成
    
    data =
      { "type"  =>  "literal",
      "value" =>  oo 
    }  

    m = Mongo::Connection.new(MONGO_SERVER,MONGO_PORT)
    db = m.db(MONGO_DB)
    db[MONGO_COLLECTION].update( {'ucode' => ss },{ '$pull' => { pp => data } } )
#    db[MONGO_COLLECTION].update( {'ucode' => ss },{ '$pull' => 'null' } )


    RDF::Reader.for(:rdfxml).new(arg2) do |reader|
      reader.each_statement do |statement|
        statement.subject.to_s =~ /([0-9a-fA-F]{32})/
        ss =  $1 # MongoDB の ucode フィールドは32桁のucodeのみ
        pp = statement.predicate.to_s
        oo = statement.object.to_s
      end
    end

    URIS.each do |i|
      pp.sub!(i[1],i[0]+'_')
    end

    # 書き込み用ハッシュ作成
    
    data =
      { "type"  =>  "literal",
      "value" =>  oo 
    }  

##    m = Mongo::Connection.new(MONGO_SERVER,MONGO_PORT)
##    db = m.db(MONGO_COLLECTION)
##    db[MONGO_COLLECTION].update( {'ucode' => ss },{ '$set' => { pp => data } } )
    db[MONGO_COLLECTION].update( {'ucode' => ss },{ '$push' => { pp => data } } )
##    db[MONGO_COLLECTION].update( {'ucode' => ss },{ '$pop'  => { pp  => -1 } } )

    head 204
  end


  #=== /api/vi/rawdata DELETE メソッドを処理する

  def destroy
    # Java 用に引数を組み立てる
    array = []

    if params[:graph].present?
      array.push("-g" + params[:graph])
    end

    if request.headers['ACCEPT'] == "text/plain"
      array.push("-n" + request.body.string)
    else request.headers['ACCEPT'] == "application/rdf+xml"
      array.push("-x" + request.body.string)
    end

    array.push("-ropendata")

    res = 0
    res =  Java::pmc::Sesamedeleteraw.results(array.to_java(:string))

    if res == 1
      head 404 and return
    end 


    # Mongo に書き込むためにSVOを抽出、加工
    ss = ''
    pp = ''
    oo = ''
    RDF::Reader.for(:rdfxml).new(request.body.string) do |reader|
      reader.each_statement do |statement|
        statement.subject.to_s =~ /([0-9a-fA-F]{32})/
        ss = $1 # MongoDB の ucode フィールドは32桁のucodeのみ
        pp = statement.predicate.to_s
        oo = statement.object.to_s
      end
    end

    URIS.each do |i|
      pp.sub!(i[1],i[0]+'_')
    end

    # 削除用ハッシュ作成
    
    data =
      { "type"  =>  "literal",
      "value" =>  oo 
    }  

    m = Mongo::Connection.new(MONGO_SERVER,MONGO_PORT)
    db = m.db(MONGO_COLLECTION)
    db[MONGO_COLLECTION].update( {'ucode' => ss },{ '$pull' => { pp => data } } )
#    db[MONGO_COLLECTION].update( {'ucode' => ss },{ '$pull' => 'null' } )
    head 204 
  end

end
