# encoding: utf-8
#
#= ResolveController
#
#Authors:: パーソナルメディア株式会社
#Copyright:: Copyrigyt (C) Personal Media Corp., 2013. All rights reserved.
#


#== /api/vi/resolve に対応したコントローラ

class ResolveController < ApplicationController

  RES_UCD      = 0x0001
  E_UIDC_OK    = 0x0000
  E_UIDC_NOSPT = 0xFFEF
  E_UIDC_PAR   = 0xFFDF
  E_UIDC_NOEXS = 0xFFCC


  UIDC_RSMODE_RESOLUTION = 0x0000 # 通常の ucode 解決応答(論理和で指定)
  UIDC_RSMODE_CACHE      = 0x0001 # キャッシュ検索による応答
  UIDC_RSMODE_CASCADE    = 0x0002 # カスケード検索による応答

  UIDC_ATTR_RS   = 0x0001 # ucode 解決サーバアドレス
  UIDC_ATTR_IS   = 0x0002 # 情報サーバ
  UIDC_ATTR_SIGS = 0x0003 # 署名サーバ
  UIDC_ATTR_USER = 0x00FF # ユーザ定義

  UIDC_DATATYPE_UCODE_128 = 0x0001 # 128bit 型 ucode
  UIDC_DATATYPE_UCODE_256 = 0x0002 # 256bit 型 ucode
  UIDC_DATATYPE_UCODE_384 = 0x0003 # 384bit 型 ucode
  UIDC_DATATYPE_UCODE_512 = 0x0004 # 512bit 型 ucode
  UIDC_DATATYPE_IPV4  = 0x0011 # IPv4 アドレス
  UIDC_DATATYPE_IPV6  = 0x0012 # IPv6 アドレス
  UIDC_DATATYPE_URL   = 0x0013 # URL
  UIDC_DATATYPE_HOST  = 0x0014 # ホスト名
  UIDC_DATATYPE_EMAIL = 0x0021 # e-mail アドレス
  UIDC_DATATYPE_PHONE = 0x0031 # 電話番号
  UIDC_DATATYPE_TXT   = 0x00FE # テキスト情報
  UIDC_DATATYPE_USER  = 0x00FF 

  #=== /api/vi/resolve/id: GET メソッドを処理する
  def show

   if !params[:id].present?
     head :bad_request and return
   end
    
    m = Mongo::Connection.new(MONGO_SERVER,MONGO_PORT)
    db = m.db(MONGO_DB)
    coll = db.collection(MONGO_COLLECTION)

    find_hash = {}

    find_hash['ucode'] = params[:id]
    find_hash['ucode'] = 'ucode_' + params[:id]  unless /^ucode_/ =~ params[:id]

    arr = []
    tmp = {}

    coll.find( find_hash ).each do |cc|
      tmp.merge!(cc)
      arr << tmp
    end

    result = {}
    result['results'] = arr
    
    respond_to do |format|
      format.json      {  render :json => JSON.pretty_generate(result) } 
      format.xml       {  render :xml  => result }
    end

  end


  #=== /api/vi/resolve POST メソッドを処理する
  def create

    data = ActiveSupport::JSON.decode(request.body.string)

    data['ucode'] = 'ucode_' + ucodeDistributor('resoleve')

    data['target'].sub!('urn:ucode:_','ucode_')
    data['target'].sub!('<','')
    data['target'].sub!('>','')

    m = Mongo::Connection.new( MONGO_SERVER ,MONGO_PORT)
    db = m.db(MONGO_DB)
    db[MONGO_DB].insert(data)
    
    
    return_hash = { 'ucode' => [] }
    return_hash['ucode'].push( '<urn:ucode:_'  + data['ucode'].sub('ucode_','') + '>'  )
    

    respond_to do |format|
      format.xml   { render :xml => return_hash }
      format.json  { render :json => return_hash }
    end

  end

  #=== /api/vi/resolve/:id PUT メソッドを処理する
  def update

    if !params[:id].present?
      header 400 and return
    end

    data = ActiveSupport::JSON.decode(request.body.string)

    ucode = params[:id]
    ucode = 'ucode_' + params[:id] unless /ucode_/ =~ params[:id]

    find_hash = {'ucode' => ucode }

    if data['target'].present?
      data['target'].sub!('urn:ucode:_','ucode_')
      data['target'].sub!('<','')
      data['target'].sub!('>','')
    end

    m = Mongo::Connection.new( MONGO_SERVER ,MONGO_PORT)
    db = m.db(MONGO_DB)

    data.each do |k,v|
      update_hash = { '$set' => {k => v} }
      db[MONGO_DB].update(find_hash, update_hash)
    end

    head 204


  end

  #=== /api/vi/resolve/:id DELETE メソッドを処理する
  def destroy

    if !params[:id].present? then
      head 400 and return # Bad Request
    end

    ucode = params[:id]
    ucode = 'ucode_' + params[:id] unless /ucode_/ =~ params[:id]

    m = Mongo::Connection.new( MONGO_SERVER ,MONGO_PORT)
    db = m.db(MONGO_DB)
    coll = db.collection(MONGO_COLLECTION)

    find_hash = { 'ucode' => ucode }

    cursor = coll.find( find_hash )

    if !cursor.has_next? then
      head :not_found and return
    end

    coll.remove( find_hash )

    head 204

  end



  def ucodeDistributor(objs=nil)
    # client = Mongo::MongoClient.new('pmc.opendata.ubin.jp', MONGO_PORT)
    # client.add_auth('ucode_distributor', 'opendata','0pendata')
    # client.apply_saved_authentication
    # db = client['ucode_distributor']
    # coll = db['ucodes']
    # doc = coll.find_one({"used"=>false})
    # if objs != nil
    #   p doc.merge! objs
    # end
    # doc['used']=true
    # coll.save doc
    # return doc["ucode"]
    return '00001C00000000000001000003000' + format("%X",rand(0x700))
  end


end
