# encoding: utf-8
#
#= RsController
#
#Authors:: パーソナルメディア株式会社
#Copyright:: Copyrigyt (C) Personal Media Corp., 2013. All rights reserved.
#


#== /api/vi/rs に対応したコントローラ

#=== 実装方法
# 以下のリクエストを投げる。
# http://rs.uidcenter.org/cgi-bin/gw?X-UIDC-UCODE=<問い合わせられたucode>&X-UIDC-GWMODE=...&X-UIDC-QUERYMASK=...&X-UIDC-QUERYATTRIBUTE=...
# HTTPのレスポンスボディに X-UIDC-DATA，X-UIDC-DATATYPE 等の値が返ってくる。
# これをレスポンスパラメータとして返却する。

class RsController < ApplicationController

  UCODE_GW = 'http://rs.uidcenter.org/cgi-bin/gw'

  RES_UCD      = 0x0001
  E_UIDC_OK    = 0x0000
  E_UIDC_NOSPT = 0xFFEF
  E_UIDC_PAR   = 0xFFDF
  E_UIDC_NOEXS = 0xFFCC

  UIDC_RSMODE_RESOLUTION = 0x0000 # 通常の ucode 解決応答(論理和で指定)
  UIDC_RSMODE_CACHE      = 0x0001 # キャッシュ検索による応答
  UIDC_RSMODE_CASCADE    = 0x0002 # カスケード検索による応答

  UIDC_ATTR_ANONYMOUS   = 0x0000 # 属性を指定しない
  UIDC_ATTR_RS   = 0x0001 # ucode 解決サーバアドレス
  UIDC_ATTR_IS   = 0x0002 # 情報サーバ
  UIDC_ATTR_SIGS = 0x0003 # 署名サーバ
  UIDC_ATTR_USER = 0x00FF # ユーザ定義
  

  UIDC_DATATYPE_UCODE_128 = 0x0001 # 128bit 型 ucode
  UIDC_DATATYPE_UCODE_256 = 0x0002 # 256bit 型 ucode
  UIDC_DATATYPE_UCODE_384 = 0x0003 # 384bit 型 ucode
  UIDC_DATATYPE_UCODE_512 = 0x0004 # 512bit 型 ucode

  UIDC_GWMODE_ARRAY = [ 'resolveall' , 'resolve' , 'redirect']
  UIDC_QUERYATTRIBUTE_ARRAY = [ UIDC_ATTR_RS , UIDC_ATTR_IS , UIDC_ATTR_SIGS , UIDC_ATTR_USER ]

  #=== /api/vi/rs/id: GET メソッドを処理する
  def show

    if !params[:id].present?
      head 400 and return # UCODEが指定されていない
    end

    ucode = params[:id].sub('ucode_','')
    gwmode = params['X-UIDC-GWMODE'].present? ? params['X-UIDC-GWMODE']  : 'resolveall'
    querymask = params['X-UIDC-QUERYMASK'].present? ? params['X-UIDC-QUERYMASK'] :  'ffffffffffffffffffffffffffffffff'
    queryattribute  = params['X-UIDC-QUERYATTRIBUTE'].present? ? params['X-UIDC-QUERYATTRIBUTE'] : UIDC_ATTR_ANONYMOUS
    header = { 'Accept' => request.headers['Accept'] }
    query = { 'X-UIDC-UCODE' => ucode, 'X-UIDC-GWMODE' => gwmode, 'X-UIDC-QUERYMASK' => querymask, 'X-UIDC-QUERY-ATTRIBUTE' => queryattribute }
    http_client = HTTPClient.new
    res = http_client.get(UCODE_GW,query,header)
    response.headers['Content-Type'] = res.contenttype
    return_hash = {}
    res.content.each_line do |l|
      tmp_array = l.split(/:/)
      return_hash.merge!( {tmp_array[0] => tmp_array[1].sub("\r\n",'') } )
      p return_hash
  end

    result = {"results" => [ return_hash] }
    

    respond_to do |format|
      format.json      {  render :json => JSON.pretty_generate(result) } 
      format.xml       {  render :xml  => result }
    end

  end

end
