# encoding: utf-8
#
#= UsersController
#
#Authors:: パーソナルメディア株式会社
#Copyright:: Copyrigyt (C) Personal Media Corp., 2013. All rights reserved.
#


require 'java'
require '/usr/java/packages/lib/ext/commons-codec-1.7.jar'
require '/usr/java/packages/lib/ext/commons-httpclient-3.1.jar'
require '/usr/java/packages/lib/ext/commons-logging-1.1.1.jar'
require '/usr/java/packages/lib/ext/log4j-1.2.14.jar'
require '/usr/java/packages/lib/ext/openrdf-sesame-2.6.10-onejar.jar'
require '/usr/java/packages/lib/ext/slf4j-api-1.7.2.jar'
require '/usr/java/packages/lib/ext/slf4j-nop-1.7.2.jar'
require '/usr/java/packages/lib/ext/sesame.jar'
require 'active_support'
require 'json/pure'
require 'json'
include RDF
#== /api/vi/datapoints に対応したコントローラ
class UsersController < ApplicationController

  #  短縮名の変換テーブル

  URIS = [
          ["fishery","http://fishery.suisancloud.net/ns/fishery#"],
          ["rdf","http://www.w3.org/1999/02/22-rdf-syntax-ns#"],
          ["rdfs","http://www.w3.org/2000/01/rdf-schema#"],
          ["fev","http://fishery.suisancloud.net/ns/fisheryevent#"],
          ["ev","http://uidcenter.org/vocab/ucr/event#"],
          ["dc","http://purl.org/dc/elements/1.1/"],
          ["uobj","http://uidcenter.org/vocab/ucr/uobj#"],
          ["foaf","http://xmlns.com/foaf/0.1/"],
          ["ug","http://uidcenter.org/vocab/ucr/ug#"],
          ["daml","http://www.daml.org/2001/03/daml+oil#"],
          ["spac","http://uidcenter.org/vocab/ucr/spac#"],
          ["vcard","http://www.w3.org/2001/vcard-rdf/3.0#"],
          ["dcterms","http://purl.org/dc/terms/"],
          ["eg","http://www.example.org/"],
          ["trans","http://uidcenter.org/vocab/ucr/transaction#"],
          ["dctype","http://purl.org/dc/dcmitype/"],
          ["ugsrv","http://uidcenter.org/vocab/ucr/ugsrv#"],
          ["xsd","http://www.w3.org/2001/XMLSchema#"],
          ["owl","http://www.w3.org/2002/07/owl#"],
          ["rss","http://purl.org/rss/1.0/"],
          ["uc","http://uidcenter.org/vocab/ucr/uc#"],
          ["ag","http://www.agri-info.jp/vocab/ag#"],
          ["ja","http://jena.hpl.hp.com/2005/11/Assembler#"],
#         ["ucode","urn:ucode:"],
          ["geof","http://www.opengis.net/def/function/geosparql/"],
          ["geo","http://www.opengis.net/ont/geosparql#"],
          ["ucr","http://www.jibaninfo.jp/ucr/ucr#"],
          ["w3cgeo","http://www.w3.org/2003/01/geo/wgs84_pos#"],
          ["sf","http://www.opengis.net/ont/sf#"],
          ["ogc","http://www.opengis.net/"],
          ["geor","http://www.opengis.net/def/rule/geosparql/"],
          ["gml","http://www.opengis.net/ont/gml#"],
          ["gs","http://www.jibaninfo.jp/vocab/ucr/gs#"],
          ["puti","http://opendata.ubin.jp/puti#"]
         ]

  #=== /api/vi/users GET メソッドを処理する
  def index

   if params.length < 3
       head :bad_request and return
   end


    m = Mongo::Connection.new(MONGO_SERVER,MONGO_PORT)
    db = m.db(MONGO_DB)
    coll = db.collection(MONGO_COLLECTION)


    find_hash = {}
    arr = []
    offset = 0
    limit = 0

    if params[:ucode].present?
      find_hash['ucode'] = params[:ucode]
    end

    if params[:offset].present?
      offset = params[:offset].to_i
    end

    if params[:limit].present?
      limit = params[:limit].to_i
    end

    # limit 値が大き過ぎる
    if limit > 500
      head 413 and return
    end

    # dc_title => tc_title.value という変形
    params.each_key do |pp|
      if pp =~ /_/
        find_hash[ pp +'.value']  = params[pp] 
      end
    end

    if limit > 0
      coll.find( find_hash ).skip(offset).limit(limit).each do |c|
        arr.push(c)
      end
    else
      coll.find( find_hash ).skip(offset).each do |c|
        arr.push(c)
      end
    end

    # Qname Conversion

    holder = {}
    tmp = {}

    arr.each do |a|
      ucode = a['ucode']
      ucode.sub!('ucode_','urn:ucode:_')
      a.delete('ucode')
      a.delete('_id')

      a.each_key do |k|
        val = a[k]
        URIS.each do |u|
          s = u[0] + '_'
          l = u[1]
          if k =~ /^#{s}/
            kk = k.sub(s,l)
            tmp[kk] = val
            break
          end
        end
      end

      holder[ucode] = tmp
    end
    
    result = {}
    result['users'] = holder
    result['remains'] = 'false'

    # 以下はxmlを吐く為の操作

    str = JSON.generate(holder)
    puts str
    str2 =  JSON.utf8_to_json_ascii(str)
    str2.gsub!(/\\"/,'"')
    str2.gsub!(/\\u/,'u')
    str2.gsub!(/u([0-9a-f][0-9a-f][0-9a-f][0-9a-f])/,'\\U\1')
    
    data = RDF::Reader.for(:json).new(str2)

    sio = StringIO.new('')
    
    RDF::Writer.for(:rdfxml).dump( data, sio, :standard_prefixes=>true )

    respond_to do |format|
      format.json      {  render :json => JSON.pretty_generate(result) } 
#     format.xml       {  render :xml  => sio.string.gsub(/U([0-9a-z]{4})/) {|s| [$1.to_i(16)].pack("U")} }
      format.xml       {  render :xml  => sio.string.gsub(/U([0-9a-z]{4})/) {|s| [$1.to_i(16)].pack("U")} }
    end
    
  end

  #=== /api/vi/users POST メソッドを処理する
  def create
    
    data = ActiveSupport::JSON.decode(request.body.string)
    
    json_str = ActiveSupport::JSON.encode(data['rdf'])
    
    URIS.each do |i|
      json_str.gsub!(i[1],i[0]+'_')
    end

    json_str =~ /urn:ucode:_\?([a-zA-Z][a-zA-Z0-9]*)/
    key_str = $1

    org_str = 'urn:ucode:_?' + key_str
    ucode   = ucodeDistributor(key_str)
    new_str = 'ucode_' + ucode
    json_str.sub!(org_str,new_str)

#    json_str.gsub!("urn:ucode:_","ucode_")
#    json_str.gsub!(/"ucode_([0-9a-fA-F]{32})":{/, '{"ucode":"ucode_\1",' )
#    json_str.sub!(/}$/,'')
#    json_str.sub!(/^{/,'')
    
    json_str.gsub!(/\\u([0-9a-z]{4})/) {|s| [$1.to_i(16)].pack("U")} 

    real_json = {}

    m = Mongo::Connection.new( MONGO_SERVER ,MONGO_PORT)
    db = m.db(MONGO_DB)
    stream = YAML.load_stream(json_str)
    stream.each do |doc|
      holder={}
      doc.each do |k,v|
        holder['ucode'] = k
        holder.merge!(v)
        db[MONGO_DB].insert(holder)
      end
    end
    return_hash = {"ucode" => { "ucode:_?" + key_str => "urn:ucode:_" + ucode}}

    json_str = ActiveSupport::JSON.encode(data['rdf'])
    new_str = 'urn:ucode:_' + ucode

    URIS.each do |i|
      json_str.gsub!(i[1],i[0]+':')
    end

    json_str.sub!(org_str,new_str)


    # Java 用に引数を組み立てる
    array = []

    if params[:graph].present?
      array.push("-g" + params[:graph])
    end

    array.push("-ropendata")

    res = 0
    RDF::Reader.for(:json).new(json_str) do |reader|
      reader.each_statement do |statement|
        xml_str  = '<?xml version="1.0" encoding="UTF-8"?>'
        xml_str += '<rdf:RDF '
        xml_str += 'xmlns:dc="http://purl.org/dc/elements/1.1/" '
        xml_str += 'xmlns:puti="http://opendata.ubin.jp/puti#" '
        xml_str += 'xmlns:ug="http://uidcenter.org/vocab/ucr/ug#" '
        xml_str += 'xmlns:w3cgeo="http://www.w3.org/2003/01/geo/wgs84_pos#" '
        xml_str += 'xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" '
        xml_str += ' >'
        xml_str += '<rdf:Description rdf:about="' + statement.subject.to_s + '">'
        xml_str += '<' + statement.predicate.to_s  + '>'
        xml_str += statement.object.to_s 
        xml_str += '</' + statement.predicate.to_s   + '>'
        xml_str += '</rdf:Description>'
        xml_str += '</rdf:RDF>'
        #       puts "\n",xml_str,"\n"
        array.push("-x" + xml_str)
        res =  Java::pmc::Sesamepost.results(array.to_java(:string))
      end
    end

    if res == 1
      head 409 and return
    end

    respond_to do |format|
      format.json  { render :json => return_hash }
      format.xml   { render :xml => return_hash  }
    end

    end


  #=== /api/vi/users/id:/property: GET メソッドを処理する
    def simpleshow

    m = Mongo::Connection.new( MONGO_SERVER ,MONGO_PORT)
    db = m.db(MONGO_DB)
    coll = db.collection(MONGO_COLLECTION)


    find_hash = {}
    arr = []

    find_hash['ucode'] = params[:id]

    coll.find( find_hash ).each do |c|
      arr.push(c)
    end

    if arr.length == 0
      head :not_found and return
    end

    # Qname Conversion

    result = []

    arr.each do |a|
      result.push( a[params[:property]][0]['value'])
    end

    respond_to do |format|
      format.json      {  render :json => JSON.pretty_generate(result) } 
      format.xml       {  render :xml  => result }
    end

  end


  #=== /api/vi/users/id: GET メソッドを処理する
  def show

    m = Mongo::Connection.new( MONGO_SERVER ,MONGO_PORT)
    db = m.db(MONGO_DB)
    coll = db.collection(MONGO_COLLECTION)


    find_hash = {}
    arr = []

    if params[:id].present?
      find_hash['ucode'] = params[:id]
    end

    coll.find( find_hash ).each do |c|
      arr.push(c)
    end

    if arr.length == 0
      head :not_found and return
    end

    # Qname Conversion

    holder = {}
    tmp = {}

    arr.each do |a|
      ucode = a['ucode']
      ucode.sub!('ucode_','urn:ucode:_')
      a.delete('ucode')
      a.delete('_id')

      a.each_key do |k|
        val = a[k]
        URIS.each do |u|
          s = u[0] + '_'
          l = u[1]
          if k =~ /^#{s}/
            kk = k.sub(s,l)
            tmp[kk] = val
            break
          end
        end
      end

      holder[ucode] = tmp
    end
    
    result = {}
    result['users'] = holder


    str = JSON.generate(holder)
    str2 =  JSON.utf8_to_json_ascii(str)
    str2.gsub!(/\\"/,'"')
    str2.gsub!(/\\u/,'u')
    str2.gsub!(/u([0-9a-f][0-9a-f][0-9a-f][0-9a-f])/,'\\U\1')

    data = RDF::Reader.for(:json).new(str2)

    sio = StringIO.new('')
    
    RDF::Writer.for(:rdfxml).dump( data, sio, :standard_prefixes=>true )

    respond_to do |format|
      format.json      {  render :json => JSON.pretty_generate(result) } 
      format.xml       {  render :xml  => sio.string.gsub(/U([0-9a-z]{4})/) {|s| [$1.to_i(16)].pack("U")} }
    end

  end



  #=== /api/vi/users PUT メソッドを処理する
  def update

    if !params[:id].present?
      head 400 and return # Bad Request
    end

    data = ActiveSupport::JSON.decode(request.body.string)

    check_ucode = params[:id].sub('ucode_','urn:ucode:_')
    
    if !data['rdf'].has_key?( check_ucode ) then
      head 400 and return # Bad Request
    end

    json_str = ActiveSupport::JSON.encode(data['rdf'])

    
    URIS.each do |i|
      json_str.gsub!(i[1],i[0]+'_')
    end

    json_str.gsub!("urn:ucode:_","ucode_")
    json_str.gsub!(/\\u([0-9a-z]{4})/) {|s| [$1.to_i(16)].pack("U")} 

    stream = YAML.load_stream(json_str)

    key = ''
    new_value = ''
    stream[0][params[:id]].each do  | k,v |
      key =  k
      new_value = v[0]['value']
    end

    org_value = ''
    m = Mongo::Connection.new( MONGO_SERVER ,MONGO_PORT)
    db = m.db(MONGO_DB)
    db[MONGO_COLLECTION].find( {'ucode' => params[:id] },{:fields => [ key ]} ).each do |row|
      row.each do |k,v|
        if k==key
          org_value = v[0]['value']
        end
      end
    end


    if org_value.nil?
      head 404  and return
    end


    stream.each do |doc|
      holder={}
      doc.each do |k,v|
        v.each do |l,m|
          db[MONGO_COLLECTION].update( {'ucode' => k },{ '$push' => { l => m.pop } } )
          db[MONGO_COLLECTION].update( {'ucode' => k },{ '$pop'  => { l  => -1 } } )
        end
      end
    end


    # Java 用に引数を組み立てる
    array = []
    array.push("-ropendata")
    xml_str  = '<?xml version="1.0" encoding="UTF-8"?>'
    xml_str += '<rdf:RDF '
    xml_str += 'xmlns:dc="http://purl.org/dc/elements/1.1/" '
    xml_str += 'xmlns:puti="http://opendata.ubin.jp/puti#" '
    xml_str += 'xmlns:ug="http://uidcenter.org/vocab/ucr/ug#" '
    xml_str += 'xmlns:w3cgeo="http://www.w3.org/2003/01/geo/wgs84_pos#" '
    xml_str += 'xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" '
    xml_str += ' >'
    xml_str += '<rdf:Description rdf:about="' + params[:id].sub('ucode_','urn:ucode:_') + '">'
    xml_str += '<' + key.sub('_',':') + '>'
    xml_str += org_value
    xml_str += '</' + key.sub('_',':')  + '>'
    xml_str += '</rdf:Description>'
    xml_str += '</rdf:RDF>'
    array.push("-x" + xml_str)
    res =  Java::pmc::Sesamedelete.results(array.to_java(:string))

    # Java 用に引数を組み立てる
    array = []
    array.push("-ropendata")
    xml_str  = '<?xml version="1.0" encoding="UTF-8"?>'
    xml_str += '<rdf:RDF '
    xml_str += 'xmlns:dc="http://purl.org/dc/elements/1.1/" '
    xml_str += 'xmlns:puti="http://opendata.ubin.jp/puti#" '
    xml_str += 'xmlns:ug="http://uidcenter.org/vocab/ucr/ug#" '
    xml_str += 'xmlns:w3cgeo="http://www.w3.org/2003/01/geo/wgs84_pos#" '
    xml_str += 'xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" '
    xml_str += ' >'
    xml_str += '<rdf:Description rdf:about="' + params[:id].sub('ucode_','urn:ucode:_') + '">'
    xml_str += '<' + key.sub('_',':') + '>'
    xml_str += new_value
    xml_str += '</' + key.sub('_',':')  + '>'
    xml_str += '</rdf:Description>'
    xml_str += '</rdf:RDF>'
    array.push("-x" + xml_str)
    res =  Java::pmc::Sesamepost.results(array.to_java(:string))

    head 204

  end

  #=== /api/vi/users PUT メソッドを処理する
  def simpleupdate

    if !params[:property].present?
      head 400 and return # Bad Request
    end

    data = ActiveSupport::JSON.decode(request.body.string )

    predicate = params[:property]
    
    org_value = ''
    m = Mongo::Connection.new( MONGO_SERVER ,MONGO_PORT)
    db = m.db(MONGO_DB)
    db[MONGO_COLLECTION].find( {'ucode' => params[:id] },{:fields => [ predicate ]} ).each do |row|
      row.each do |k,v|
        if k==predicate
          org_value = v[0]['value']
        end
      end
    end


    if org_value.nil?
      head 404  and return
    end

    new_value = data.pop

    db[MONGO_COLLECTION].update( {'ucode' => params[:id] },{ '$push' => { predicate => {"type" => "literal", "value"=> new_value  } } } )
    db[MONGO_COLLECTION].update( {'ucode' => params[:id] },{ '$pop'  => { predicate => -1 } } )

    data = ActiveSupport::JSON.decode(request.body.string )

    # Java 用に引数を組み立てる
    array = []
    array.push("-ropendata")
    xml_str  = '<?xml version="1.0" encoding="UTF-8"?>'
    xml_str += '<rdf:RDF '
    xml_str += 'xmlns:dc="http://purl.org/dc/elements/1.1/" '
    xml_str += 'xmlns:puti="http://opendata.ubin.jp/puti#" '
    xml_str += 'xmlns:ug="http://uidcenter.org/vocab/ucr/ug#" '
    xml_str += 'xmlns:w3cgeo="http://www.w3.org/2003/01/geo/wgs84_pos#" '
    xml_str += 'xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" '
    xml_str += ' >'
    xml_str += '<rdf:Description rdf:about="' + params[:id].sub('ucode_','urn:ucode:_') + '">'
    xml_str += '<' + params[:property].sub('_',':') + '>'
    xml_str += org_value
    xml_str += '</' + params[:property].sub('_',':')  + '>'
    xml_str += '</rdf:Description>'
    xml_str += '</rdf:RDF>'
    array.push("-x" + xml_str)
    res =  Java::pmc::Sesamedelete.results(array.to_java(:string))

    # Java 用に引数を組み立てる
    array = []
    array.push("-ropendata")
    xml_str  = '<?xml version="1.0" encoding="UTF-8"?>'
    xml_str += '<rdf:RDF '
    xml_str += 'xmlns:dc="http://purl.org/dc/elements/1.1/" '
    xml_str += 'xmlns:puti="http://opendata.ubin.jp/puti#" '
    xml_str += 'xmlns:ug="http://uidcenter.org/vocab/ucr/ug#" '
    xml_str += 'xmlns:w3cgeo="http://www.w3.org/2003/01/geo/wgs84_pos#" '
    xml_str += 'xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" '
    xml_str += ' >'
    xml_str += '<rdf:Description rdf:about="' + params[:id].sub('ucode_','urn:ucode:_') + '">'
    xml_str += '<' + params[:property].sub('_',':') + '>'
    xml_str += new_value
    xml_str += '</' + params[:property].sub('_',':')  + '>'
    xml_str += '</rdf:Description>'
    xml_str += '</rdf:RDF>'
    array.push("-x" + xml_str)
    res =  Java::pmc::Sesamepost.results(array.to_java(:string))


    head 204

  end

  #=== /api/vi/users DELETE メソッドを処理する
  def destroy
    
    if !params[:id].present? then
      head 400 and return # Bad Request
    end

    m = Mongo::Connection.new( MONGO_SERVER ,MONGO_PORT)
    db = m.db(MONGO_DB)
    coll = db.collection(MONGO_COLLECTION)

    find_hash = { 'ucode' => params[:id] }

    cursor = coll.find( find_hash )


    if !cursor.has_next? then
      head :not_found and return
    end

    json_str = ''

    coll.find( find_hash ).each do |i|
      json_str = i.to_s
    end

    coll.remove( find_hash )

    arr = eval(json_str)

    holder = {}
    tmp = {}

    ucode = arr['ucode']
    ucode.sub!('ucode_','urn:ucode:_')
    arr.delete('ucode')
    arr.delete('_id')

    arr.each_key do |k|
      val = arr[k]
      kk = k.sub('_',':')
      tmp[kk] = val
    end

    holder[ucode] = tmp


    # Java 用に引数を組み立てる

    RDF::Reader.for(:json).new(holder.to_json) do |reader|
      reader.each_statement do |statement|
        array = []
        array.push("-ropendata")
        xml_str  = '<?xml version="1.0" encoding="UTF-8"?>'
        xml_str += '<rdf:RDF '
        xml_str += 'xmlns:dc="http://purl.org/dc/elements/1.1/" '
        xml_str += 'xmlns:puti="http://opendata.ubin.jp/puti#" '
        xml_str += 'xmlns:ug="http://uidcenter.org/vocab/ucr/ug#" '
        xml_str += 'xmlns:w3cgeo="http://www.w3.org/2003/01/geo/wgs84_pos#" '
        xml_str += 'xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" '
        xml_str += ' >'
        xml_str += '<rdf:Description rdf:about="' + statement.subject.to_s + '">'
        xml_str += '<' + statement.predicate.to_s  + '>'
        xml_str += statement.object.to_s 
        xml_str += '</' + statement.predicate.to_s   + '>'
        xml_str += '</rdf:Description>'
        xml_str += '</rdf:RDF>'
        array.push("-x" + xml_str)
        puts array.inspect
        res =  Java::pmc::Sesamedelete.results(array.to_java(:string))
        puts res
      end
    end
    

    head :ok and return

  end

  #=== /api/vi/users DELETE メソッドを処理する
  def simpledestroy

    if ( !params[:id].present?  or !params[:property].present?  )
      head 400 and return # Bad Request
    end

    m = Mongo::Connection.new( MONGO_SERVER ,MONGO_PORT)
    db = m.db(MONGO_DB)
    coll = db.collection(MONGO_COLLECTION)
    
    find_hash = { 'ucode' => params[:id] }
    field_hash = { :fields =>  params[:property]  }
    result_hash = { '$unset' => { params[:property] => 1 }}

    cursor = coll.find( find_hash ,field_hash)

    if !cursor.has_next? then

      head :not_found and return

    end
      
    # > db.opendata.update({'ucode':"ucode_00001C000000000000010000030008BF"},{$unset:{ 'dc_title':1}})
    coll.update( find_hash, result_hash )

    json_str = ''

    coll.find( find_hash ).each do |i|
      json_str = i.to_s
    end

    holder = {}
    tmp = {}

    arr = eval(json_str)

    ucode = arr['ucode']
    ucode.sub!('ucode_','urn:ucode:_')
    arr.delete('ucode')
    arr.delete('_id')

    arr.each_key do |k|
      val = arr[k]
      if k == params[:property]
        kk = k.sub('_',':')
        tmp[kk] = val
      end
    end

    holder[ucode] = tmp


    # Java 用に引数を組み立てる

    RDF::Reader.for(:json).new(holder.to_json) do |reader|
      reader.each_statement do |statement|
        array = []
        array.push("-ropendata")
        xml_str  = '<?xml version="1.0" encoding="UTF-8"?>'
        xml_str += '<rdf:RDF '
        xml_str += 'xmlns:dc="http://purl.org/dc/elements/1.1/" '
        xml_str += 'xmlns:puti="http://opendata.ubin.jp/puti#" '
        xml_str += 'xmlns:ug="http://uidcenter.org/vocab/ucr/ug#" '
        xml_str += 'xmlns:w3cgeo="http://www.w3.org/2003/01/geo/wgs84_pos#" '
        xml_str += 'xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" '
        xml_str += ' >'
        xml_str += '<rdf:Description rdf:about="' + statement.subject.to_s + '">'
        xml_str += '<' + statement.predicate.to_s  + '>'
        xml_str += statement.object.to_s 
        xml_str += '</' + statement.predicate.to_s   + '>'
        xml_str += '</rdf:Description>'
        xml_str += '</rdf:RDF>'
        array.push("-x" + xml_str)
        puts array.inspect
        res =  Java::pmc::Sesamedelete.results(array.to_java(:string))
        puts res
      end
    end

    head :ok
      
  end


  def ucodeDistributor(objs=nil)
    # client = Mongo::MongoClient.new('pmc.opendata.ubin.jp', MONGO_PORT)
    # client.add_auth('ucode_distributor', 'opendata','0pendata')
    # client.apply_saved_authentication
    # db = client['ucode_distributor']
    # coll = db['ucodes']
    # doc = coll.find_one({"used"=>false})
    # if objs != nil
    #   p doc.merge! objs
    # end
    # doc['used']=true
    # coll.save doc
    # return doc["ucode"]
    return '00001C00000000000001000003000' + format("%X",rand(0x700))
  end

  
end
