package pmc;

import java.io.*;

import org.openrdf.model.*;
import org.openrdf.model.impl.*;
import org.openrdf.repository.Repository;
import org.openrdf.repository.RepositoryConnection;
import org.openrdf.repository.RepositoryException;
import org.openrdf.repository.http.HTTPRepository;
import org.openrdf.rio.RDFWriter;
import org.openrdf.rio.RDFHandlerException;
import org.openrdf.rio.ntriples.NTriplesWriter;
import org.openrdf.rio.rdfxml.RDFXMLWriter;
import org.openrdf.rio.n3.N3Writer;

public class Sesameget
{
   private static String SESAME_SERVER =
       "http://sesame.opendata.ubin.jp:8080/openrdf-sesame";
   
   public static void get(
       Resource subject, URI predicate, Value object, String repository,
       RDFWriter w, Resource context )
       throws RDFHandlerException, RepositoryException {
      Repository myRepository = new HTTPRepository(SESAME_SERVER, repository);
      myRepository.initialize();
      RepositoryConnection conn = myRepository.getConnection();
      conn.exportStatements(subject, predicate, object, true, w, context);
      conn.close();
   }
   public static void main( String args[] ) throws Exception
   {
      // <cmd> -s(subject) -p(predicate) -o(object/uri) -O(object/Literal)
      //       -n(ntriples output file) -x(RDF/XML output file) -r(repositoryID)
      //       -g(URI of Named Graph)

      String repository = "opendata";
      Resource subject = null;
      URI predicate = null;
      Value object = null;
      URI context = null;
      RDFWriter w = null;
      StringWriter str = new StringWriter();

      for (int i=0; i<args.length; i++) {
         if (args[i].charAt(0) == '-') {
            switch(args[i].charAt(1)) {
             case 's':
               subject = new URIImpl(args[i].substring(2));
               break;
             case 'p':
               predicate = new URIImpl(args[i].substring(2));
               break;
             case 'o':
               object = new URIImpl(args[i].substring(2));
               break;
             case 'O':
               object = new LiteralImpl(args[i].substring(2));
               break;
             case 'g':
               context = new URIImpl(args[i].substring(2));
               break;
             case 'r':
               repository = args[i].substring(2);
               break;
            case '3':
                w = new N3Writer(str);
                break;
            case 'n':
                w = new NTriplesWriter(str);
                break;
            case 'x':
                w = new RDFXMLWriter(str);
                break;
            }
         }
      }
      if (w == null) {
         w =  new NTriplesWriter(System.out);
      }
      get(subject, predicate, object, repository, w, context);
      System.out.println( str.toString() );
   }

   public static String results( String args[] ) throws Exception
   {
      // <cmd> -s(subject) -p(predicate) -o(object/uri) -O(object/Literal)
      //       -n(ntriples output file) -x(RDF/XML output file) -r(repositoryID)
      //       -g(URI of Named Graph)

      String repository = "opendata";
      Resource subject = null;
      URI predicate = null;
      Value object = null;
      URI context = null;
      RDFWriter w = null;
      StringWriter str = new StringWriter();

      for (int i=0; i<args.length; i++) {
         if (args[i].charAt(0) == '-') {
            switch(args[i].charAt(1)) {
             case 's':
               subject = new URIImpl(args[i].substring(2));
               break;
             case 'p':
               predicate = new URIImpl(args[i].substring(2));
               break;
             case 'o':
               object = new URIImpl(args[i].substring(2));
               break;
             case 'O':
               object = new LiteralImpl(args[i].substring(2));
               break;
             case 'g':
               context = new URIImpl(args[i].substring(2));
               break;
             case 'r':
               repository = args[i].substring(2);
               break;
            case '3':
                w = new N3Writer(str);
                break;
            case 'n':
                w = new NTriplesWriter(str);
                break;
            case 'x':
                w = new RDFXMLWriter(str);
                break;
            }
         }
      }
      if (w == null) {
         w =  new NTriplesWriter(System.out);
      }
      get(subject, predicate, object, repository, w, context);
      return str.toString() ;
   }
}

