package pmc;

import java.io.*;
import java.util.*;

import org.openrdf.model.*;
import org.openrdf.model.impl.*;
import org.openrdf.repository.Repository;
import org.openrdf.repository.RepositoryConnection;
import org.openrdf.repository.RepositoryException;
import org.openrdf.repository.http.HTTPRepository;
import org.openrdf.rio.RDFFormat;
import org.openrdf.rio.RDFParseException;
import org.openrdf.rio.RDFParser;
import org.openrdf.rio.RDFParseException;
import org.openrdf.rio.RDFHandlerException;
import org.openrdf.rio.ntriples.NTriplesWriter;
import org.openrdf.rio.rdfxml.RDFXMLWriter;
import org.openrdf.rio.ntriples.NTriplesParser;
import org.openrdf.rio.rdfxml.RDFXMLParser;
import org.openrdf.rio.helpers.StatementCollector;

public class Sesamepost
{
   private static String SESAME_SERVER =
       "http://sesame.opendata.ubin.jp:8080/openrdf-sesame";
   
    public static int post( Reader in, Reader in2,  RDFFormat format, RDFParser parser, String repository, Resource context )
        throws RDFParseException, RepositoryException, IOException,RDFHandlerException {
        int b = 0;
        Repository myRepository = new HTTPRepository(SESAME_SERVER, repository);
        myRepository.initialize();
        RepositoryConnection conn = myRepository.getConnection();

        StatementCollector hdr = new StatementCollector(new ArrayList<Statement>());
        parser.setRDFHandler(hdr);
        parser.parse(in, "");

        for (Iterator<Statement> i = hdr.getStatements().iterator(); i.hasNext();){

            Statement st = (Statement)i.next();
            if ( conn.hasStatement(st,true, context) ){
                b = 1;
                break;
            }
        }


        if (b == 0){
            conn.add(in2, null, format, context);
        } 
        conn.close();
        return b;
    }

    public static void main( String args[] ) throws Exception
        {
            // <cmd> -n(ntriples input file) -x(RDF/XML inputt file) -r(repositoryID)
            //       -g(URI of Named Graph)
            Reader in = null;
            URI context = null;
            RDFFormat format = RDFFormat.RDFXML;
            String repository = "opendata";
            RDFParser parser = new RDFXMLParser();
            for (int i=0; i<args.length; i++) {
                if (args[i].charAt(0) == '-') {
                    switch(args[i].charAt(1)) {
                    case 'n':
                        in = new BufferedReader(new StringReader(args[i].substring(2)));
                        parser = new NTriplesParser();
                        format = RDFFormat.NTRIPLES;
                        break;
                    case 'x':
                        in = new BufferedReader(new StringReader(args[i].substring(2)));
                        format = RDFFormat.RDFXML;
                        parser = new RDFXMLParser();
                        break;
                    case 'r':
                        repository = args[i].substring(2);
                        break;
                    case 'g':
                        context = new URIImpl(args[i].substring(2));
                        break;
                    }
                }
            }
            if (in == null) {
                in = new BufferedReader(new InputStreamReader(System.in));
            }

            post(in, in, format, parser, repository, context);
        }

    public static int results( String args[] ) throws Exception
        {
            // <cmd> -n(ntriples input file) -x(RDF/XML inputt file) -r(repositoryID)
            //       -g(URI of Named Graph)
            Reader in = null;
            Reader in2 = null;
            URI context = null;
            RDFFormat format = RDFFormat.RDFXML;
            String repository = "opendata";
            RDFParser parser = new RDFXMLParser();
            for (int i=0; i<args.length; i++) {
                if (args[i].charAt(0) == '-') {
                    switch(args[i].charAt(1)) {
                    case 'n':
                        in = new BufferedReader(new StringReader(args[i].substring(2)));
                        in2 = new BufferedReader(new StringReader(args[i].substring(2)));
                        parser = new NTriplesParser();
                        format = RDFFormat.NTRIPLES;
                        break;
                    case 'x':
                        in = new BufferedReader(new StringReader(args[i].substring(2)));
                        in2 = new BufferedReader(new StringReader(args[i].substring(2)));
                        format = RDFFormat.RDFXML;
                        parser = new RDFXMLParser();
                        break;
                    case 'r':
                        repository = args[i].substring(2);
                        break;
                    case 'g':
                        context = new URIImpl(args[i].substring(2));
                        break;
                    }
                }
            }
            if (in == null) {
                in = new BufferedReader(new InputStreamReader(System.in));
            }

            return post(in, in2, format, parser, repository, context);
        }

}
